"""PAYDUNYA Invoice"""
from . import Payment
from collections import namedtuple

InvoiceItem = namedtuple('InvoiceItem', 'name quantity unit_price \
                             total_price description')


class Invoice(Payment):
    """Payment invoice"""

    def __init__(self, store=None):
        """Create an invoice

        Accepts list of store object as initial parameter and
        a dictionary of tokens for accessing the PAYDUNYA API
        """
        self.cancel_url = None
        self.return_url = None
        self.callback_url = None
        self.description = None
        self.items = {}
        self.total_amount = 0
        self.custom_data = {}
        self.taxes = {}
        super(Invoice, self).__init__()
        if store:
            self.store = store

    def create(self, items=[], taxes=[], custom_data=[]):
        """Adds the items to the invoice

        Format of 'items':
        [
         InvoiceItem(
             name="VIP Ticket",
             quantity= 2,
             unit_price= "3500",
             total_price= "7000",
             description= "VIP Tickets for the Party"
          }
        ,...
        ]
        """
        self.add_items(items)
        self.add_taxes(taxes)
        self.add_custom_data(custom_data)
        return self._process('checkout-invoice/create', self._prepare_data)

    def confirm(self, token=None):
        """Returns the status of the invoice

        STATUSES: pending, completed, cancelled
        """
        _token = token if token else self._response.get("token")
        return self._process('checkout-invoice/confirm/' + str(_token))

    def add_taxes(self, taxes):
        """Appends the data to the 'taxes' key in the request object

        'taxes' should be in format: [("tax_name", "tax_amount")]
        For example:
        [("Other TAX", 700), ("VAT", 5000)]
        """
        # fixme: how to resolve duplicate tax names
        _idx = len(self.taxes)  # current index to prevent overwriting
        for idx, tax in enumerate(taxes):
            tax_key = "tax_" + str(idx + _idx)
            self.taxes[tax_key] = {"name": tax[0], "amount": tax[1]}

    def add_custom_data(self, data=[]):
        """Adds the data to teh custom data sent to the server

        data format: [("phone_brand", Motorola V3"), ("model", "65456AH23")]
        """
        self.custom_data.update(dict(data))

    def add_item(self, item):
        """Updates the list of items in the current transaction"""
        _idx = len(self.items)
        self.items.update({"item_" + str(_idx + 1): item})

    def add_items(self, items):
        for item in items:
            self.add_item(item)

    @property
    def _prepare_data(self):
        """Formats the data in the current transaction for processing"""
        total_amount = self.total_amount or self.calculate_total_amt()
        self._data = {
            "invoice": {
                "items": self.__encode_items(self.items),
                "taxes": self.taxes,
                "total_amount": total_amount,
                "description": self.description,
            },
            "store": self.store.info,
            "custom_data": self.custom_data,
            "actions": {
                "cancel_url": self.cancel_url,
                "return_url": self.return_url,
                "callback_url": self.callback_url
            }
        }
        return self._data

    def calculate_total_amt(self, items={}):
        """Returns the total amount/cost of items in the current invoice"""
        _items = items.items() or self.items.items()
        return sum(float(x[1].total_price) for x in _items)

    def __encode_items(self, items):
        """Encodes the InvoiceItems into a JSON serializable format

        items = [('item_1',InvoiceItem(name='VIP Ticket', quantity=2,
                             unit_price='3500', total_price='7000',
                             description='VIP Tickets for party')),...]
        """
        xs = [item._asdict() for (_key, item) in items.items()]
        return list(map(lambda x: dict(zip(x.keys(), x.values())), xs))



# access keys
PAYDUNYA_ACCESS_TOKENS = {
    'PAYDUNYA-MASTER-KEY': "5b9f531a-fbb8-487a-8045-3b4c7ac5acee",
    'PAYDUNYA-PRIVATE-KEY': "test_private_oGslgmzSNL3RSkjlsnPOsZZg9IA",
    'PAYDUNYA-TOKEN': "ff1d576409b2587cc1c2"
}



"""PAYDUNYA DirectPay"""
from . import Payment

class DirectPay(Payment):
    """Directpay processing class

    Receipient account_info format:
    '{
       "account_alias" : "774563209",
       "amount" : 215000
    }'
    """
    def __init__(self, account_alias=None, amount=None):
        self.transaction = {
            'account_alias': account_alias,
            'amount': amount
        }
        super(DirectPay, self).__init__()

    def process(self, transaction=None):
        """Process the transaction"""
        return self._process('direct-pay/credit-account',
                             transaction or self.transaction)



"""PAYDUNYA Onsite Payments Request"""
from . import Payment


class OPR(Payment):
    """Onsite Payment Request"""
    def __init__(self, data={}, store=None):
        self._opr_data = self._build_opr_data(data, store)
        super(OPR, self).__init__()
        if store:
            self.store = store

    def _build_opr_data(self, data, store):
        """Returns a well formatted OPR data"""
        return {
            "invoice_data": {
                "invoice": {
                    "total_amount": data.get("total_amount"),
                    "description": data.get("description")
                },
                "store": store.info
            },
            "opr_data": {
                "account_alias": data.get("account_alias")
            }
        }

    def create(self, data={}, store=None):
        """Initiazes an OPR

        First step in the OPR process is to create the OPR request.
        Returns the OPR token
        """
        _store = store or self.store
        _data = self._build_opr_data(data, _store) if data else self._opr_data
        return self._process('opr/create', _data)

    def charge(self, data):
        """Second stage of an OPR request"""
        token = data.get("token", self._response["token"])
        data = {
            "token": token,
            "confirm_token": data.get("confirm_token")
        }
        return self._process('opr/charge', data)



import unittest
import paydunya
from . import PAYDUNYA_ACCESS_TOKENS

paydunya.debug = True
paydunya.api_keys = PAYDUNYA_ACCESS_TOKENS


class TestOPR(unittest.TestCase):
    def setUp(self):
        self.opr_data = {'account_alias': '774563209',
                         'description': 'Hello World',
                         'total_amount': 345}
        store = paydunya.Store(name="Event Shop")
        self.opr = paydunya.OPR(self.opr_data, store)

    def tearDown(self):
        self.opr = None

    def test_opr_create(self):
        status, resp = self.opr.create()
        self.assertTrue(status)

        status, _ = self.opr.create(self.opr_data)
        self.assertTrue(status)

    def test_opr_charge(self):
        status, response = self.opr.create()
        token = response['token']
        status, _ = self.opr.charge({'token': token,
                                     'confirm_token': "56Y8"})
        # request should because the token and
        # confirm_token combination are wrong
        self.assertFalse(status)

if __name__ == '__main__':
    unittest.main()



import unittest
import paydunya
from . import PAYDUNYA_ACCESS_TOKENS

paydunya.debug = True
paydunya.api_keys = PAYDUNYA_ACCESS_TOKENS


class Invoice(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.store = paydunya.Store(name="Event Shop")
        self.items = [
            paydunya.InvoiceItem(
                name="VIP Ticket",
                quantity=2,
                unit_price="35000",
                total_price="70000",
                description="VIP Tickets for the Youssou N'dour Event"
            )
        ]
        self.invoice = paydunya.Invoice(self.store)
        self.invoice.add_items(self.items * 10)
        # taxes are (key,value) pairs
        self.invoice.add_taxes([("Other TAX", 700), ("VAT", 5000)])
        self.invoice.add_custom_data([("phone_brand", "Motorola V3"),
                                      ("model", "65456AH23")])

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        self.invoice = None

    def test_create_invoice(self):
        status, _ = self.invoice.create()
        self.assertTrue(status)

    def test_add_items(self):
        _items = len(self.invoice.items)
        self.invoice.add_items(self.items * 2)
        self.assertTrue(len(self.invoice.items) > _items)

    def test_add_taxes(self):
        _taxes = len(self.invoice.taxes)
        self.invoice.add_taxes([("Foo_TAX", 9)])
        self.assertTrue(len(self.invoice.taxes) > _taxes)

    def test_add_custom_data(self):
        _data = len(self.invoice.custom_data)
        self.invoice.add_custom_data([("Category", "Sports")])
        self.assertTrue(len(self.invoice.custom_data) > _data)



from setuptools import setup, Command
from unittest import TextTestRunner, TestLoader
from glob import glob
from os.path import splitext, basename, join as pjoin
try:
    from os.path import walk
except ImportError:
    from os import walk
import os


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        self._dir = os.getcwd()

    def finalize_options(self):
        pass

    def run(self):
        '''
        Finds all the tests modules in tests/, and runs them.
        '''
        testfiles = []
        for t in glob(pjoin(self._dir, 'tests', '*.py')):
            if not t.endswith('__init__.py'):
                testfiles.append('.'.join(
                    ['tests', splitext(basename(t))[0]])
                )
        import sys
        ROOT = os.path.dirname(os.getcwd())
        PAYDUNYA_LIBS = os.path.join(ROOT, "paydunya")
        sys.path.append(PAYDUNYA_LIBS)
        tests = TestLoader().loadTestsFromNames(testfiles)
        t = TextTestRunner(verbosity=1)
        t.run(tests)


class CleanCommand(Command):
    """Recursively Delete all compile python modules"""
    user_options = []

    def initialize_options(self):
        self._clean_me = []
        for root, dirs, files in os.walk('.'):
            for f in files:
                if f.endswith('.pyc'):
                    self._clean_me.append(pjoin(root, f))

    def finalize_options(self):
        pass

    def run(self):
        for clean_me in self._clean_me:
            try:
                os.unlink(clean_me)
            except:
                pass


def readme(filename='README.rst'):
    with open('README.rst') as f:
        text = f.read()
    f.close()
    return text

setup(
    name='paydunya',
    version=__import__('paydunya').__version__,
    author='PAYDUNYA',
    author_email='paydunya@paydunya.com',
    packages=['paydunya'],
    cmdclass={'test': TestCommand, 'clean': CleanCommand},
    scripts=[],
    url='https://github.com/paydunya/paydunya-python',
    license='LICENSE.txt',
    keywords="paydunya mobile money payments",
    description='PAYDUNYA Python client library',
    long_description=readme('README.rst'),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=['requests >=2.0'],
)



import unittest
import paydunya
from . import PAYDUNYA_ACCESS_TOKENS

paydunya.debug = True
paydunya.api_keys = PAYDUNYA_ACCESS_TOKENS


class TestGeneral(unittest.TestCase):
    """General/Miscellaneous tests"""
    def setUp(self):
        # Your PAYDUNYA developer tokens
        self.store = paydunya.Store(name="Fabrice Accessoires")
        self.opr_data = {'total_amount': 215000,
                         'description': "Samsung Galaxy S6",
                         "account_alias": "774563209"}
        self.opr = paydunya.OPR(self.opr_data, self.store)

    def tearDown(self):
        self.opr = None
        self.store = None
        self.opr_data = None

    def test_rsc_endpoints(self):
        endpoint = 'checkout-invoice/confirm/test_98567JGF'
        url = self.opr.get_rsc_endpoint(endpoint)
        self.assertTrue(url.startswith('https') and url.endswith(endpoint))

    def test_add_headers(self):
        header = {'Foo': 'Bar'}
        self.opr.add_header(header)
        self.assertTrue("Foo" in self.opr.headers.keys())
        self.assertFalse('FooBar' in self.opr.headers.keys())


if __name__ == '__main__':
    unittest.main()



"""PAYDUNYA

PAYDUNYA Python client library.
Modules implemented: DirectPay, Invoice, and OPR
"""

__version__ = '1.0.5'
__author__ = "PAYDUNYA <paydunya@paydunya.com>"

import sys
import requests
try:
    import simplejson as json
except ImportError:
    import json

# runs in LIVE mode by defaults
debug = False
api_keys = {}

# PAYDUNYA HTTP API version
API_VERSION = 'v1'

SERVER = "app.paydunya.com"

# Sandbox Endpoint
SANDBOX_ENDPOINT = "https://%s/sandbox-api/%s/" % (SERVER, API_VERSION)

# Live Endpoint
LIVE_ENDPOINT = "https://%s/api/%s/" % (SERVER, API_VERSION)

# user-agent
PAYDUNYA_USER_AGENT = "paydunya-python/v%s" % __version__

# fixme: find a better way of 'self' referencing
__MODULE__ = sys.modules[__name__]


class PaydunyaError(Exception):
    """Base Exception class"""
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Store(object):
    """PAYDUNYA Store

    Creates a store object for PAYDUNYA transactions
    """
    def __init__(self, **kwargs):
        self.name = kwargs.get('name', None)
        self.tagline = kwargs.get('tagline', None)
        self.postal_address = kwargs.get('postal_address', None)
        self.phone_number = kwargs.get('phone_number', None)
        self.website_url = kwargs.get('website_url', None)
        self.logo_url = kwargs.get('logo_url', None)

    @property
    def info(self):
        """Returns the store information

        What this does is simply return the store object's attributes
        """
        return self.__dict__


class Payment(object):
    """Base class for other PAYDUNYA classes"""

    def __init__(self):
        """Base class for all the other payment libraries"""
        # request headers
        self._headers = {
            'User-Agent': PAYDUNYA_USER_AGENT,
            "Content-Type": "application/json"
        }
        # response object
        self._response = None
        # data to send to server
        self._data = None
        self.store = Store(name=None)

    def _process(self, resource=None, data={}):
        """Processes the current transaction

        Sends an HTTP request to the PAYDUNYA API server
        """
        # use object's data if no data is passed
        _data = data or self._data
        rsc_url = self.get_rsc_endpoint(resource)
        if _data:
            req = requests.post(rsc_url, data=json.dumps(_data),
                                headers=self.headers)
        else:
            req = requests.get(rsc_url, params=_data,
                               headers=self.headers)
        if req.status_code == 200:
            self._response = json.loads(req.text)
            if int(self._response['response_code']) == 00:
                return (True, self._response)
            else:
                return (False, self._response['response_text'])
        else:
            return (500, "Request Failed")

    @property
    def headers(self):
        """Returns the client's Request headers"""
        return dict(self._config, **self._headers)

    def add_header(self, header):
        """Add a custom HTTP header to the client's request headers"""
        if type(header) is dict:
            self._headers.update(header)
        else:
            raise ValueError(
                "Dictionary expected, got '%s' instead" % type(header)
            )

    def get_rsc_endpoint(self, rsc):
        """Returns the HTTP API URL for current payment transaction"""
        if self.debug:
            return SANDBOX_ENDPOINT + rsc
        return LIVE_ENDPOINT + rsc

    @property
    def debug(self):
        """Returns the current transaction mode"""
        return __MODULE__.debug

    @property
    def _config(self):
        _m = __MODULE__
        return {
            'PAYDUNYA-MASTER-KEY': _m.api_keys.get('PAYDUNYA-MASTER-KEY'),
            'PAYDUNYA-PRIVATE-KEY': _m.api_keys.get('PAYDUNYA-PRIVATE-KEY'),
            'PAYDUNYA-TOKEN': _m.api_keys.get('PAYDUNYA-TOKEN')
        }


# moved here so the modules that depend on the 'Payment' class will work
from .invoice import Invoice, InvoiceItem
from .direct_payments import DirectPay
from .opr import OPR

__all__ = [
    Store.__name__,
    Payment.__name__,
    Invoice.__name__,
    InvoiceItem.__name__,
    DirectPay.__name__,
    OPR.__name__
]
